from RoboidControl.Messages.IMessage import IMessage

from typing import Union
import logging
logger = logging.getLogger(__name__)

## A message communicating the network ID for that participant
class NetworkIdMsg(IMessage):
    ## The message ID
    id: int = 0xA1
    ## The length of the message in bytes
    length: int = 2
    
    def __init__(self, arg1: Union[bytes, int]):
        """! Create a NetworkIdMsg
        @param arg1 Either:
        - bytes: contianing the serialized data; or
        - int: containgint the network_id 
        """
        self.network_id: int = 0
        if isinstance(arg1, bytes):
            buffer = bytearray(arg1)
            self.network_id = buffer[1]
        else:
            self.network_id = int(arg1)

    def Serialize(self, buffer_ref: list[bytearray]) -> int:    
        """!
        @copydoc RoboidControl::Messages::IMessage::IMessage::Serialize
        """ 
        logger.debug(f'Send NetworkIdMsg {self.network_id}')

        buffer: bytearray = buffer_ref[0]
        last:int = NetworkIdMsg.length
        buffer[0:last] = [
            NetworkIdMsg.id,
            self.network_id
        ]
        return NetworkIdMsg.length

