from RoboidControl.Messages.IMessage import IMessage
from RoboidControl.Thing import Thing

from typing import Optional, Union
import logging
logger = logging.getLogger(__name__)

class ThingMsg(IMessage):
    """! Message providing generic details about a Thing
    """
    ## The message ID
    id = 0x80
    ## The length of the message
    length = 5

    def __init__(self, arg1: Union[bytes, int], thing: Optional[Thing]=None):
        """! Create a ModelUrlMsg
        @param arg1 Either:
        - bytes: containing the serialized data; or
        - int: containing the network_id 
        @param thing The thing for which the data is communicated
        """        
        if thing is None:
            buffer = bytes(arg1)
            ## The network ID of the thing
            self.network_id: int = buffer[1]
            ## The ID of the thing
            self.thing_id: int = buffer[2]
            ## The type of thing
            self.thing_type: int = buffer[3]
            ## The ID of the parent thing in the hierarchy. This is zero for root things
            self.parent_id: int = buffer[4]
        else:
            ## The network ID of the thing
            self.network_id = int(arg1)
            ## The ID of the thing
            self.thing_id = thing.id
            ## The type of thing
            self.thing_type = thing.type
            ## The ID of the parent thing in the hierarchy. This is zero for root things
            if thing.parent is not None:
                self.parent_id = thing.parent.id
            else:
                self.parent_id = 0

    def Serialize(self, buffer_ref: list[bytearray]):
        """!
        @copydoc RoboidControl::Messages::IMessage::IMessage::Serialize
        """ 
      
        logger.debug(f'Send ThingMsg [{self.network_id}/{self.thing_id}] {self.thing_type} {self.parent_id}')

        buffer: bytearray = buffer_ref[0]
        buffer[0:ThingMsg.length] = [
            ThingMsg.id,
            self.network_id,
            self.thing_id,
            self.thing_type,
            self.parent_id
        ]
        return ThingMsg.length